import "Turbine";
import "Turbine.UI";
import "Turbine.Gameplay";
import "Deusdictum.UI";
import "Ingaras.HotswapSlot.Settings";
import "Ingaras.HotswapSlot.Options";

function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

language=1;
locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end

settings = Settings( );

HotswapBar = class( Turbine.UI.Window );
HotswapBar.spacing = 2;

function HotswapBar:Constructor( equipmentType, bar_id, set_data )
	Turbine.UI.Window.Constructor( self );
	
	if set_data == nil then
		self:SetPosition( ( Turbine.UI.Display:GetWidth() - 250 ) / 3, ( Turbine.UI.Display:GetHeight() - 250 ) / 3 );
		self.vertical = false;
	else
		self:SetPosition( set_data.position.x, set_data.position.y );
		self.vertical = set_data.vertical;	
	end
	self:SetBackColor( Turbine.UI.Color( 0, 1,1,1 ) );
	
	self.id = bar_id;
	self.type = equipmentType;
	local barType = "Unknown";
	if( equipmentType == Turbine.Gameplay.Equipment.Class ) then
		barType = "Class";
	elseif( equipmentType == Turbine.Gameplay.Equipment.PrimaryWeapon ) then
		barType = "Weapon";
	elseif( equipmentType == Turbine.Gameplay.Equipment.RangedWeapon ) then
		barType = "Ranged";
	end 
	
	self:SetText( "HS ".. barType .." ".. self.id );

	self.clickstate = 0;
	local localplayer = Turbine.Gameplay.LocalPlayer:GetInstance();
	local equipment = localplayer.GetEquipment();
	local onItemEquipped = function( sender, args )
		if(args.Index == self.type ) then
			local hotswapShortcut = self.quickSlot1:GetShortcut();
			if(self.clickstate >= 1) then
				hotswapShortcut = self.shortcut1;
			end
			local hotswapItem = hotswapShortcut:GetItem();
			
			if (self.clickstate >= 1 and hotswapItem:GetName() ~= args.Item:GetName()) then
				self.quickSlot1:SetShortcut( self.shortcut1 );
				self.overlay:SetVisible(true);
				self.overlaytext:SetVisible(true);
				self.quickSlot3:SetVisible(false);
				self.clickstate = 0;
			elseif(hotswapItem:GetName() == args.Item:GetName()) then
				self.shortcut1 = self.quickSlot1:GetShortcut();
				self.quickSlot1:SetShortcut( self.quickSlot2:GetShortcut() );
				self.overlay:SetVisible(false);
				self.overlaytext:SetVisible(false);
				self.quickSlot3:SetVisible(true);
				self.clickstate = 1;			
			end
		end
	end
	AddCallback(equipment, "ItemEquipped", onItemEquipped);
	
	local effects = localplayer:GetEffects();
	local onClick = function( sender, args )
		local effect = effects:Get(args['Index']);
		if(self.clickstate == 1 and effect:IsDebuff() == false ) then
			self.quickSlot1:SetShortcut( self.quickSlot3:GetShortcut() );
			self.quickSlot3:SetVisible(false);
			self.clickstate = 2;
		end
	end
	AddCallback(effects, "EffectAdded", onClick);
		
	self.quickSlot1 = Turbine.UI.Lotro.Quickslot();
	self.quickSlot1:SetParent( self );
	self.quickSlot1:SetPosition(0,0);
	
	self.quickSlot2 = Turbine.UI.Lotro.Quickslot();
	self.quickSlot2:SetParent( self );
	if self.vertical then
		self.quickSlot2:SetPosition( 0, (self.quickSlot1:GetWidth() + self.spacing));
	else
		self.quickSlot2:SetPosition( (self.quickSlot1:GetWidth() + self.spacing), 0);	
	end
	self.quickSlot2:SetVisible(false);
	
	self.quickSlot3 = Turbine.UI.Lotro.Quickslot();
	self.quickSlot3:SetParent( self );
	if self.vertical then
		self.quickSlot3:SetPosition( 0, (self.quickSlot1:GetWidth() + self.spacing));
	else
		self.quickSlot3:SetPosition( (self.quickSlot1:GetWidth() + self.spacing), 0);	
	end
	self.quickSlot3:SetVisible(false);
	
	if set_data ~= nil then
		if(set_data.qs1.data ~= "") then
			self.quickSlot1:SetShortcut(settings:LoadShortcut(set_data.qs1));
		end
		if(set_data.qs2.data ~= "") then
			self.quickSlot2:SetShortcut(settings:LoadShortcut(set_data.qs2));
		end
		if(set_data.qs3.data ~= "") then
			self.quickSlot3:SetShortcut(settings:LoadShortcut(set_data.qs3));		
		end
	end
	
	self.overlay = Turbine.UI.Lotro.Quickslot();
	self.overlay:SetParent( self );
	self.overlay:SetMouseVisible(false);
	self.overlay:SetPosition(0,0);
	self.overlay:SetShortcut(self.quickSlot2:GetShortcut());
	self.overlay:SetZOrder( self.quickSlot1:GetZOrder() + 1 );
	local onSCC = function( sender, args )
		self.overlay:SetShortcut( self.quickSlot2:GetShortcut() );
		if self.DragBar:IsDraggable() then
			self.overlay:SetVisible(false);
		end
	end
	AddCallback(self.quickSlot2, "ShortcutChanged", onSCC);
	
	self.overlaytext = Turbine.UI.Label();
	self.overlaytext:SetParent( self );
	self.overlaytext:SetText("SWAP");
	self.overlaytext:SetZOrder( self.overlay:GetZOrder() + 1 );
	self.overlaytext:SetWidth( self.overlay:GetWidth() );
	self.overlaytext:SetHeight( self.overlay:GetHeight() );
	self.overlaytext:SetMouseVisible(false);
	self.overlaytext:SetTextAlignment( Turbine.UI.ContentAlignment.BottomCenter );
	
	if(self.vertical) then
		self:SetSize( self.quickSlot1:GetHeight(), 3 * (self.quickSlot1:GetWidth() + self.spacing) );	
	else
		self:SetSize( 3 * (self.quickSlot1:GetWidth() + self.spacing), self.quickSlot1:GetHeight() );
	end
	
	self.DragBar = Deusdictum.UI.DragBar( self, self:GetText() );
	self.DragBar:SetBarOnTop( true );
	self.DragBar:SetRotator( true );
	
	self:SetVisible( true );
end

function HotswapBar:DragEnable()
	if self.vertical then
		self.quickSlot3:SetPosition( 0, 2*(self.quickSlot1:GetWidth() + self.spacing));
	else
		self.quickSlot3:SetPosition( 2*(self.quickSlot1:GetWidth() + self.spacing), 0);	
	end
	self.quickSlot2:SetVisible(true);
	self.quickSlot3:SetVisible(true);
	self.overlay:SetVisible(false);
	self.overlaytext:SetVisible(false);
end

function HotswapBar:DragDisable()
	if self.vertical then
		self.quickSlot3:SetPosition( 0, (self.quickSlot1:GetWidth() + self.spacing));
	else
		self.quickSlot3:SetPosition( (self.quickSlot1:GetWidth() + self.spacing), 0);	
	end
	self.quickSlot2:SetVisible(false);
	self.quickSlot3:SetVisible(false);
	self.overlay:SetVisible(true);
	self.overlaytext:SetVisible(true);
	
	settings:Store();
end

function HotswapBar:Rotate( args )
	local y, x = self:GetSize();
	self:SetSize(x, y);
	self.DragBar:RecalculateSize();
	
	local py, px = self.quickSlot2:GetPosition();
	self.quickSlot2:SetPosition( px, py );
	local py, px = self.quickSlot3:GetPosition();
	self.quickSlot3:SetPosition( px, py );
	
	if self.vertical == true then
		self.vertical = false;
	else
		self.vertical = true;
	end
end

settings.bars = {};
for key, value in pairs (settings.data.quickslots) do
	if value ~= nil then
		settings.bars[key] = HotswapBar( value.type, key, value );
	end
end
if table.getn( settings.bars ) == 0 then
	settings.bars[1] = HotswapBar( Turbine.Gameplay.Equipment.Class, 1, nil );
end

optionsPanel = Options();
plugin.GetOptionsPanel = function( self )
	return optionsPanel;
end

local onUnload = function( sender, args )
	settings:Store();
end
AddCallback( plugin, "Unload", onUnload );